(function () {
    function onMessageExternal(request, sender, responseCallback) {
        console.logv('on message external:', (request && request.eventName) || 'unknown');
        if (request && request.eventName) {
            var response = {};
            if (request.eventName == Events.External.Actions.Ajax_CopyToClipboard) {
                var ajaxOptions = $.extend({}, request.ajaxOptions, true);

			$.ajax(ajaxOptions).done(function (value) {
			    copyText(value);
			    response.Success = true;
			}).fail(function (error) {
			    response.Success = false;
			});
		}
		else if (request.eventName == Events.External.Actions.GetVersion) {
			// version info is returned by default
		}
		else if (request.eventName == Events.External.Actions.IsAvailable) {
			response.isAvailable = true;
		}

            // handle callback w/ response info
            if (typeof (responseCallback) == 'function') {
                var defaultResponse = {
                    version: chrome.runtime.getManifest().version
                };
                var responseObj = $.extend({}, defaultResponse, response, true);
                responseCallback(responseObj);
            }
        }
    }

    function onMessage(request, sender, responseCallback) {
        console.logv('on message:', (request && request.eventName) || 'unknown');
        if (request && request.eventName) {
            if (request.eventName == Events.Actions.NeedsPageAction) {
                console.logv('received needsPageAction');
                placePageAction(sender);
            }
            else if (request.eventName == Events.Settings.Get || request.eventName == Events.Settings.Reset) {
                console.logv('received ' + request.eventName);
                if (request.eventName == Events.Settings.Reset) {
                    chrome.storage.sync.remove('settings', function () {
                        Settings.get(responseCallback);
                    });
                }
                else {
                    Settings.get(responseCallback);
                }
                return true;
            }
            else if (request.eventName == Events.Settings.Save) {
                console.logv('received saveSettings');
                Settings.save(request.settings, responseCallback);
                return true;
            }
            else if (request.eventName == Events.Notifications.RaiseNotification) {
                console.logv('received raiseNotification');
                console.logv('sender', sender);
                Settings.get(function (settings) {
                    if (settings.notificationsEnabled) {
                        Notification.raise(request.notification.ID, request.notification.options, request.notification.data, (sender.tab || {}).id, settings, request.forceNotification);
                    }
                    else {
                        console.warn('Cannot raise notification: Notifications are not enabled.');
                    }
                });

                return true;
            }
            else if (request.eventName == Events.Actions.OpenTab) {
                Omnibox.openTab(request.url, request.disposition || 'newForegroundTab', request.existingTabPattern);
            }
            else if (request.eventName == Events.Actions.NavigateTab) {
                Omnibox.navigateTab(request.url, request.protocol || 'https', request.disposition || 'newForegroundTab', request.existingTabPattern);
            }
            else if (request.eventName == Events.Actions.SetClipboard) {
                var text = request.copyText ? request.copyText : '';

                copyText(text);

                responseCallback();

                return true;
            }
        }
    };

    // AJAX events
    function checkForSupportRequests() {
        // TODO: create AJAX call to check for support requests
    }

    function onAlarm(alarm) {
        if (alarm.name.startsWith(snoozedString)) {
            var event = alarm.name.substring(snoozedString.length);
            if (event.startsWith('supportRequestNotification')) {
                console.logv('Alarm firing: ' + alarm.name);
                Settings.get(function (settings) {
                    if (settings
                        && settings.enabled
                        && settings.notificationsEnabled
                        && settings.supportRequestNotificationsEnabled) {

                        var notificationID = alarm.name;
                        getNotificationData(function (notificationData) {
                            var data = notificationData[notificationID];
                            Notification.raiseSupportRequest(event, data, settings);
                        });
                    }
                    else {
                        clearAlarm(alarm.name, true);
                    }
                });
            }
            else {
                clearAlarm(alarm.name);
            }
        }
        else if (alarm.name == "checkForSupportRequests") {
            // TODO: if, between 7am - 7pm, check for support requests
            //       else delay alarm until 7am
        }
        else if (alarm.name == "standUpAlarm") {
            console.logv('Alarm firing: ' + alarm.name);
            Settings.get(function (settings) {
                if (settings
                    && settings.enabled
                    && settings.notificationsEnabled
                    && settings.standUpNotificationsEnabled) {
                    var notificationOptions = {
                        title: 'Time to get up!',
                        message: "You've been sitting for a while - it's time to stand up for a few minutes.",
                        contextMessage: 'Notifications are configurable in settings.',
                        isClickable: true
                    };

                    Notification.raise('standUpNotification', notificationOptions, null, null, settings);
                }
                else {
                    clearAlarm(alarm.name, true);
                }
            });
        }
        else if (alarm.name == "resetSignalRConnection") {
            try {
                if (SignalR && !SignalR.isConnected()) {
                    SignalR.startConnection();
                }
            }
            catch (ex) { }
        }
    }

    function clearAlarm(alarmName, disabled) {
        chrome.alarms.clear(alarmName, function () {
            console.logv('Cleared alarm ' + alarmName + (disabled ? ' -- notification disabled (Settings)' : ''));
        });
    }


    function startSignalR() {
        try {
            if (SignalR) {
                SignalR.startConnection();
            }
        }
        catch (ex) { }
        /*
        try {
            // reset SignalR connection every X mins
            var recycleSignalRConnectionTimeoutInMinutes = 30;
            chrome.alarms.create('resetSignalRConnection', { delayInMinutes: recycleSignalRConnectionTimeoutInMinutes, periodInMinutes: recycleSignalRConnectionTimeoutInMinutes });
            console.logv('Alarm', 'set "resetSignalRConnection"');
        }
        catch (ex) { }
        */
    }

    function startup(event) {
        startSignalR();
    }

    function onInstalled(details) {
        if (details.reason == 'install'
            || (details.reason == 'update'
                && details.previousVersion < chrome.runtime.getManifest().version)) {
            chrome.tabs.create({ 'url': chrome.runtime.getURL('info.html') }, function () { });
        }

        Settings.get(function (settings) {
            if (settings
                && settings.enabled
                && settings.notificationsEnabled
                && settings.standUpNotificationsEnabled) {
                var interval = settings.standUpNotificationInterval || 45;
                chrome.alarms.create('standUpAlarm', { delayInMinutes: interval, periodInMinutes: interval });
                console.logv('created standUpAlarm');
            }
            else {
                clearAlarm('standUpAlarm');
            }
        });

        //startSignalR();
        //reloadAllTabs();
    }

    // Event Listeners
    chrome.runtime.onStartup.addListener(startup);
    chrome.runtime.onInstalled.addListener(startup);
    chrome.runtime.onInstalled.addListener(onInstalled);
    chrome.runtime.onMessage.addListener(onMessage);
    chrome.runtime.onMessageExternal.addListener(onMessageExternal);
    chrome.runtime.onUpdateAvailable.addListener(function (details) {
        console.log("updating to version " + details.version);
        chrome.runtime.reload();
    });
    chrome.runtime.requestUpdateCheck(function (status) {
        if (status == chrome.runtime.RequestUpdateCheckStatus.UPDATE_AVAILABLE) {
            console.log("update pending...");
            chrome.runtime.reload();
        } else if (status == chrome.runtime.RequestUpdateCheckStatus.NO_UPDATE) {
            console.logv("no update found");
        } else if (status == chrome.runtime.RequestUpdateCheckStatus.THROTTLED) {
            console.logv("Oops, I'm asking too frequently - I need to back off.");
        }
    });

    chrome.alarms.onAlarm.addListener(onAlarm);

    /*
    chrome.commands.onCommand.addListener(function(command) {
        console.logv('Command: ' + command);
    	
        if (command == 'toggle-enabled') {
            Settings.get(function(curSettings) {
                console.logv('Toggling 'enabled'. Previous value: ' + curSettings.enabled + '. New value: ' + !curSettings.enabled);
            	
                if (curSettings.enabled) {
                    chrome.notifications.getAll(function(currentNotifications) {
                        for (var key in currentNotifications) {
                            console.logv('Clearing notification: ' + key);
                            chrome.notifications.clear(key, function() {});						
                        }
                    });
                }
    
                curSettings.enabled = !curSettings.enabled;
                Settings.save(curSettings, function() { reloadAllTabs(); });
            });
        }
    });
    */
})();

function sendTestNotification2() {
    var notificationData = {
        link: '/PersonTimeSearch.aspx'
    };

    Settings.get(function (settings) {
        var notificationID = 'generalNotification';
        var notificationOptions = notificationData.notificationOptions;

        Notification.raise(notificationID, notificationOptions, notificationData, null, settings, notificationData.forceNotification);
    });
}

function sendTestNotification() {
    var defaultNotificationOptions = {
        iconUrl: chrome.runtime.getURL('assets/img/PioneerRx-Icon-128.png'),
        imageUrl: 'http://www.pioneerrx.com/App_Themes/NewTechBlue/Images/Employee_Images/Adam_Fausett.jpg',
        type: 'image',
        title: 'PioneerRx',
        message: 'There\'s something you should see',
        isClickable: true
    };

    var notificationOptions = {
        title: 'Testing Options',
        message: '<b>This</b> is a test'
    };
    notificationOptions = $.extend({}, defaultNotificationOptions, notificationOptions);

    var notificationID = 'testNotification_' + Date.now();
    chrome.notifications.create(notificationID, notificationOptions, function () { });
}
