(function (exports) {
    var MIN_LOADING = 0; // 0.75 seconds
    var TOKEN_VALIDATION_TIMEOUT = 1000 * 60 * 60 * 24; // 1 day

    var currentSettings;

    function goToMain() {
        window.location = 'main.html';
    }

    function valid() {
        var invalid = false;
        var focused = false;

        var txtUsername = document.getElementById('txtUsername');
        var txtPassword = document.getElementById('txtPassword');

        var divUsernameGroup = document.getElementById('divUsernameGroup');
        var divPasswordGroup = document.getElementById('divPasswordGroup');

        var successClassName = 'form-group';
        var errorClassName = 'form-group has-error';

        // validate username 
        if ($.trim(txtUsername.value) == '') {
            invalid = true;
            divUsernameGroup.className = errorClassName;
            if (!focused) {
                focused = true;
                txtUsername.focus();
                addError('Username is required');
            }
        }
        else {
            divUsernameGroup.className = successClassName;
        }

        // validate password
        if ($.trim(txtPassword.value) == '') {
            invalid = true;
            divPasswordGroup.className = errorClassName;
            if (!focused) {
                focused = true;
                txtPassword.focus();
                addError('Password is required');
            }
        }
        else {
            divPasswordGroup.className = successClassName;
        }

        return !invalid;
    }

    function login() {
        if (valid()) {
            var loginName = txtUsername.value;
            var password = txtPassword.value;

            var timestamp;
            $.ajax({
                type: 'GET',
                url: 'https://www.pioneerrx.com/Services/SecurityService.svc/GetSecurityToken?l=' + loginName + '&p=' + encodeURIComponent(password),
                contentType: 'application/json; charset=utf-8',
                dataType: 'json',
                beforeSend: function () {
                    timestamp = Date.now();

                    document.getElementById('modal_title').innerHTML = 'Signing in&hellip;';
                    $('#pleaseWaitDialog').modal();
                }
            })
                .done(function (data) {
                    if (data) {
                        var settings = {
                            enabled: true,
                            notificationsEnabled: true,
                            loginName: loginName,
                            securityToken: data,
                            securityTokenTimestamp: timestamp
                        };

                        saveToken(settings);
                    }
                    else {
                        clearToken('Incorrect username or password');
                    }
                })
                .fail(function (data) {
                    clearToken('Incorrect username or password');
                });
        }
    }

    var updatedSecurityToken = false;
    function updateSecurityToken(token) {
        var timestamp;
        $.ajax({
            type: 'GET',
            url: 'https://www.pioneerrx.com/api/PioneerApi/LoggedOnUserSecurityToken',
            contentType: 'application/json; charset=utf-8',
            dataType: 'json',
            headers: { 'prx_ss_value': '892djAS82j@0Dk90@KpO0-92jpoidauf0' },
            beforeSend: function () {
                timestamp = Date.now();
            },
            success: function (data) {
                if (data) {
                    var settings = {
                        enabled: true,
                        notificationsEnabled: true,
                        securityToken: data,
                        securityTokenTimestamp: timestamp
                    };

                    saveToken(settings);
                }
                else {
                    updateSecurityToken_Error();
                }
            },
            error: function (data) {
                updateSecurityToken_Error();
            },
            complete: function () {
                $('#pleaseWaitDialog').modal('hide');
                updatedSecurityToken = true;
            }
        });
    }

    function updateSecurityToken_Error() {
        clearToken('<h3>Authentication failed</h3><p> - Please ensure that you are logged into Support Center.</p>');
        try {
            var background = chrome.extension.getBackgroundPage();
            if (background && background.SignalR) {
                background.SignalR.stopConnection();
            }
        }
        catch (ex) { }
    }

    function clearToken(errorMessage) {
        var settings = {
            enabled: false,
            notificationsEnabled: false,
            securityToken: null,
            securityTokenTimestamp: null
        };

        saveToken(settings, function () {
            if (errorMessage) {
                addError(errorMessage);
            }
        });

        $('.loading').hide();
    }

    function saveToken(settings, callback) {
        callback = callback || goToMain;
        settings = $.extend({}, currentSettings, settings);

        chrome.runtime.sendMessage({ eventName: Events.Settings.Save, settings: settings }, function (response) {
            var action;
            if (response && response.success != undefined && (response.success || response.errorMessage)) {
                action = function () {
                    console.logv('Successfully saved Security Token.');
                };
            }
            else {
                action = function () {
                    addError('Error saving Security Token - please try again.');
                };
            }

            setTimeout(function () {
                $('#pleaseWaitDialog').modal('hide');

                if (action) {
                    action();
                }

                try {
                    var background = chrome.extension.getBackgroundPage();
                    if (background && background.SignalR && !background.SignalR.isConnected()) {
                        background.SignalR.startConnection();
                    }
                }
                catch (ex) { }

                if (callback && typeof (callback) == 'function') {
                    callback();
                }
            }, MIN_LOADING - (Date.now() - (settings.securityTokenTimestamp || Date.now())));
        });
    }

    function addError(errorMessage) {
        var errorSummary = $('#errorSummary');
        if (errorMessage) {
            $('#pleaseWaitDialog').modal('hide');
            errorSummary.append('<li>' + errorMessage + '</li>');
        }
    }

    function clearErrors() {
        var errorSummary = $('#errorSummary');
        errorSummary.empty();
    }

    document.addEventListener('DOMContentLoaded', function () {
        setTimeout(function () {
            if (!updatedSecurityToken) {
                document.getElementById('modal_title').innerHTML = 'Validating Credentials&hellip;';
                $('#pleaseWaitDialog').modal();
            }
        }, 500);

        chrome.runtime.sendMessage({ eventName: Events.Settings.Get, }, function (settings) {
            currentSettings = settings;

            updateSecurityToken(currentSettings.securityToken);
        });
    }, false);
})(window);