(function (exports) {
    var MAX_GROWINGTEXTAREA_HEIGHT = 200;
    var MIN_LOADING = 800; // 0.8 seconds
    var FEEDBACK_TIMEOUT = 1750;

    var BASE_ERROR = '<span class="glyphicon glyphicon-exclamation-sign"></span>';
    var SUCCESS_GLYPH = '<span class="glyphicon glyphicon-ok-sign" style="padding-left: 5px;"></span>';
    var ERROR_GLYPH = '<span class="glyphicon glyphicon-exclamation-sign" style="padding-left: 5px;"></span>';

    var currentSettings;
    var textareaSize, input, isSubmitting;

    function handleEnterKey(e) {
        clearErrors();

        /*
        var enterKey = 13;
        if (e.which == enterKey && e.shiftKey) {
            if ($(textareaSize).height() < MAX_GROWINGTEXTAREA_HEIGHT) {
                textareaSize.innerHTML = input.value + '\n\n';
            }
        }
        else if (e.which == enterKey) {
            processDailyStatusDetail(e);
        }
        */

        try {
            var enterKey = 13;
            if ((e.which == enterKey && !e.shiftKey)
                && (input.value && input.value.trim().length > 0)) {
                processDailyStatusDetail(e);
            }
            else if (e.which == enterKey && !e.shiftKey) {
                input.value = '';
            }

            var timeout = (e.type == 'paste' ? 100 : 0);

            setTimeout(function () {
                var newlineCount = (input.value.match(/\n/g) || []).length;
                var brCount = (input.value.match(/<br \/>/g) || []).length;

                newlineCount += brCount;

                var temp = '';
                for (var i = 0; i < newlineCount; ++i) {
                    if ($(textareaSize).height() < MAX_GROWINGTEXTAREA_HEIGHT) {
                        temp += (newlineCount == 1 ? '\n\n' : '\n');
                    }
                }

                textareaSize.innerHTML = input.value + temp;
            }, timeout);
        } catch (ex) { }
    }

    function valid() {
        var invalid = false;

        var lblError = document.getElementById('lblDailyStatusDetailError');
        var txtDailyStatusDetail = document.getElementById('txtDailyStatusDetail');

        var divDailyStatusDetailGroup = document.getElementById('divDailyStatusDetailGroup');

        var successClassName = 'form-group';
        var errorClassName = 'form-group has-error';

        if ($.trim(txtDailyStatusDetail.value) == '') {
            invalid = true;
            divDailyStatusDetailGroup.className = errorClassName;
            $(lblError).removeClass('hide');
            lblError.innerHTML = 'Entry cannot be blank';
            txtDailyStatusDetail.focus();
        }
        else {
            divDailyStatusDetailGroup.className = successClassName;
        }

        return !invalid;
    }

    function addDailyStatus() {
        if (valid()) {
            try {
                var lblError = document.getElementById('lblDailyStatusDetailError');
                var txtDailyStatusDetail = document.getElementById('txtDailyStatusDetail');

                var divDailyStatusDetailGroup = document.getElementById('divDailyStatusDetailGroup');
                var btnDailyStatusDetailSubmit = document.getElementById('btnDailyStatusDetailSubmit');

                var normalClassName = 'form-group';
                var successClassName = 'form-group has-success';
                var errorClassName = 'form-group has-error';

                var token = currentSettings.securityToken;
                var timestamp = Date.now();

                var postData = {
                    "EntryToAdd": txtDailyStatusDetail.value.split('\n').join('<br />')
                };

                var debugUrl = 'https://prxafausett.pioneerrx.com/pioneerrxinternet/api/DailyStatus/Add';
                var productionUrl = 'https://www.pioneerrx.com/api/DailyStatus/Add';

                var success, errorMessage;
                $.ajax({
                    type: 'POST',
                    url: (useProduction ? productionUrl : debugUrl),
                    contentType: 'application/json; charset=utf-8',
                    dataType: 'json',
                    data: JSON.stringify(postData),
                    beforeSend: function () {
                        isSubmitting = true;
                        success = false;

                        $('#pleaseWaitDialog').modal();
                    }
                })
                    .done(function (data) {
                        success = true;

                        if (data && data.CustomErrorMessage) {
                            success = false;
                            errorMessage = data.CustomErrorMessage;
                        }
                    })
                    .fail(function (data) {
                        errorMessage = 'Error adding Daily Status';
                    })
                    .always(function () {
                        setTimeout(function () {
                            $('#pleaseWaitDialog').modal('hide');

                            var backToNormal = function () {
                                isSubmitting = false;

                                divDailyStatusDetailGroup.className = normalClassName;
                                btnDailyStatusDetailSubmit.disabled = false;
                                btnDailyStatusDetailSubmit.innerHTML = 'Submit';
                            };

                            if (success) {
                                divDailyStatusDetailGroup.className = successClassName;
                                btnDailyStatusDetailSubmit.innerHTML = 'Success' + SUCCESS_GLYPH;

                                setTimeout(function () {
                                    backToNormal();
                                }, FEEDBACK_TIMEOUT);

                                // clear textbox
                                txtDailyStatusDetail.value = '';
                                textareaSize.innerHTML = '';

                                $(txtDailyStatusDetail).height(34);
                                $(textareaSize).height(34);
                            }
                            else {
                                divDailyStatusDetailGroup.className = errorClassName;
                                btnDailyStatusDetailSubmit.innerHTML = 'Error' + ERROR_GLYPH;

                                setTimeout(function () {
                                    backToNormal();
                                }, FEEDBACK_TIMEOUT);
                            }

                            if (errorMessage) {
                                $(lblError).removeClass('hide');
                                lblError.innerHTML = errorMessage;
                            }
                        }, MIN_LOADING - (Date.now() - timestamp));

                        txtDailyStatusDetail.focus();
                    });
            }
            catch (ex) {
                isSubmitting = false;
            }
        }
    }

    function clearErrors() {
        var lblError = document.getElementById('lblDailyStatusDetailError');
        var divDailyStatusDetailGroup = document.getElementById('divDailyStatusDetailGroup');

        lblError.innerHTML = '';
        $(lblError).addClass('hide');
        divDailyStatusDetailGroup.className = 'form-group';
    }

    function processDailyStatusDetail(e) {
        e.preventDefault();
        if (!isSubmitting) {
            clearErrors();
            if (valid()) {
                addDailyStatus();
            }
        }
    }

    document.addEventListener('DOMContentLoaded', function () {
        //$('.loading').hide();

        var textContainer = document.querySelector('.growing-textarea-wrapper');
        textareaSize = textContainer.querySelector('.growing-textarea-size');
        input = textContainer.querySelector('.growing-textarea');

        //input.addEventListener('input', autoSize);
        input.addEventListener('keydown', handleEnterKey);

        var savedInput = localStorage["dsInput"];
        if (savedInput && savedInput !== undefined) {
            input.value = savedInput;
            input.select();
        }

        var lnkTimeClock = document.getElementById('lnkTimeClock');
        var lnkPayrollPortal = document.getElementById('lnkPayrollPortal');

        var btnDailyStatusDetailSubmit = document.getElementById('btnDailyStatusDetailSubmit');
        var formDailyStatusDetailSubmit = document.getElementById('formDailyStatusDetail');

        var dailyStatusWrapper = document.getElementById('divDailyStatusWrapper');
        var employeeSearchWrapper = document.getElementById('employeeListWrapper');

        chrome.runtime.sendMessage({ eventName: Events.Settings.Get, }, function (settings) {
            currentSettings = settings;

            if (settings.showDailyStatusQuickAdd) {
                $(dailyStatusWrapper).removeClass('hide');
            }

            if (settings.showEmployeeSearch) {
                $(employeeSearchWrapper).removeClass('hide');

                (!settings.showDailyStatusQuickAdd && $('#txtEmployeeSearch').focus() || $(dailyStatusWrapper).append('<hr />'));
            }

            $('#btnProfile').html((settings.loginName || 'Menu') + ' <span class="caret"></span>');

            setTimeout(function () {
                $('.loading').fadeOut();
            }, 100);
        });

        $('.fixed-header #header_logo').click(function (e) {
            window.open('http://www.pioneerrx.com', '_blank');
        });

        formDailyStatusDetailSubmit.addEventListener('submit', function (e) {
            processDailyStatusDetail(e);
        }, false);

        btnDailyStatusDetailSubmit.addEventListener('click', function (e) {
            processDailyStatusDetail(e);
        }, false);

        lnkTimeClock.addEventListener('click', function (e) {
            chrome.runtime.sendMessage({ eventName: Events.Actions.NavigateTab, url: '/PersonTimeSearch.aspx', existingTabPattern: 'PersonTimeSearch.aspx' }, null);
        }, false);

        lnkPayrollPortal.addEventListener('click', function (e) {
            chrome.runtime.sendMessage({ eventName: Events.Actions.OpenTab, url: 'https://eselfserve.com', existingTabPattern: 'https://eselfserve.com/*' }, null);
        }, false);
    });
})(window);


addEventListener('unload', function (event) {
	localStorage["dsInput"] = input.value;
});