var logVerbose = false;
var logDebug = false;
var useProduction = true;

console.logv = function() { if (logVerbose) { console.log.apply(this, arguments); } }
console.logd = function() { if (logDebug) { console.log.apply(this, arguments); } }

var snoozedString = 'snoozedNotification_';

// ASP date handlers
Date.reISO = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/;
Date.reMsAjax = /^\/Date\((d|-|.*)\)[\/|\\]$/;

Date.toAjaxString = function (date) {
    if (date != null) {
        return "\/Date(" + date.valueOf() + ")\/";
    }
    else if (this != null && this.constructor == Date) {
        return "\/Date(" + this.valueOf() + ")\/";
    }
    else {
        return null;
    }
}

Date.parseAjaxString = function (dtString) {
    if (dtString != null) {
        var a = Date.reISO.exec(dtString);
        if (a)
            return new Date(Date.UTC(+a[1], +a[2] - 1, +a[3],
                                     +a[4], +a[5], +a[6]));
        a = Date.reMsAjax.exec(dtString);
        if (a) {
            var b = a[1].split(/[-,.]/);
            return new Date(+b[0]);
        }
        return null; 
    }
    else {
        return null;
    }
}

function logout(currentSettings) {
	// clear stored daily status input
	localStorage["dsInput"] = '';
	
	var settings = {
		enabled: false,
		notificationsEnabled: false,
		securityToken: null,
		securityTokenTimestamp: null
	};
	
	settings = $.extend({}, currentSettings, settings);

    chrome.runtime.sendMessage({ eventName: Events.Settings.Save, settings: settings }, function (response) {
		if (response && response.success != undefined && (response.success || response.errorMessage)) {
			console.logv('successfully logged out.');
			try {
				window.location = 'login.html';
			}
			catch (ex) {
				console.error('Error switching to login screen.');
			}
		}
	});

	// close SignalR connection when logging out
	try {
		var background = chrome.extension.getBackgroundPage();
		if (background && background.SignalR) {
			background.SignalR.stopConnection();
		}
	}
	catch (ex) { }
}

// generate a random string in guid format
function newGuid() {
  function s4() {
    return Math.floor((1 + Math.random()) * 0x10000)
      .toString(16)
      .substring(1);
  }
  return s4() + s4() + '-' + s4() + '-' + s4() + '-' +
    s4() + '-' + s4() + s4() + s4();
}

function checkForUpdate() {
	chrome.runtime.requestUpdateCheck(function(status, details) {
		console.logv('Update request response: Status: ' + status + ', Details: ' + JSON.stringify(details));
	});
}

function placePageAction(sender) {
	chrome.pageAction.show(sender.tab.id);
}

function copyText(text) {
	// Adapted from http://stackoverflow.com/a/5255791/448
	var textarea = document.createElement('textarea');
	var body = document.getElementsByTagName('body')[0];
	body.appendChild(textarea);

	// now we put the message in the textarea
	textarea.value = text;

	// and copy the text from the textarea
	textarea.select();
	document.execCommand('copy', false, null);
	body.removeChild(textarea);
	console.logv('copied ' + text);
}

function reloadAllTabs(callback) {
	for (var j = 0; j < matchingUrls.length; j++) {
		chrome.tabs.query({ 'url': matchingUrls[j] }, function(tabs) {
			if (tabs && tabs.length > 0) {
				for (var i = 0; i < tabs.length; i++) {
					console.logv('reloading matching tab ' + tabs[i].id + ' with url ' + tabs[i].url);
					chrome.tabs.reload(tabs[i].id);
				}
			}

			if (typeof callback !== 'undefined') {
				callback();
			}
		});
	}
}

function popupNamedWindow(url, guid, width, height)
{
    if(!guid)
        guid = '';

    var windowName = guid.replace(/-/g, '');

    return popupWindow(url, width, height, windowName);
}

function popupWindow(url, width, height, target)  
{  
    var win;
    win = window.open(  
         url,  
         target,  
         'status=1,toolbar=0,menubar=0,location=0,resizable=1,scrollbars=1,height=' + height + ',width=' + width);

    // required for addressing a bug in Chrome 3.0 to 4.0
    if (typeof (win) != 'undefined' && !win.closed) {
        win.blur();
    }
    
    win.focus();
    
    return win;
}
