var Notification = (function () {
    var notificationTimeout = 5 * 1000;
    var lastNotificationTimestamp;
    var lastNotificationTabID;

    var defaultNotificationData = {};

    var defaultNotificationOptions = {
        iconUrl: chrome.runtime.getURL('assets/img/PioneerRx-Icon-128.png'),
        type: 'basic',
        title: 'PioneerRx Notification',
        message: 'There\'s something you should see',
        isClickable: false
    };

    function raiseSupportRequestNotification(notificationID, data, settings) {
        var notificationOptions = {
            type: 'basic',
            title: 'New Support Request',
            message: data.comment,
            contextMessage: 'Issue#: ' + data.issueNumber + ' - ' + data.subject,
            isClickable: true,
            buttons: [
                { title: 'View Issue', iconUrl: chrome.runtime.getURL('assets/img/icon-eye-32.png') },
                { title: 'Snooze', iconUrl: chrome.runtime.getURL('assets/img/icon-clock-grey-32.png') }
            ]
        };

        raiseNotification(notificationID, notificationOptions, data, null, settings);
    }

    function raiseNotification(notificationID, notificationOptions, data, tabID, settings, forceNotification) {
        if (!forceNotification
            && lastNotificationTimestamp
            && new Date() < new Date(lastNotificationTimestamp.getTime() + notificationTimeout)) {
            console.logv('notification timeout not met - no notification raised. last update time: '
                + lastNotificationTimestamp + '. waiting until '
                + new Date(lastNotificationTimestamp.getTime() + notificationTimeout));
        }
        else {
            notificationID = notificationID + '_' + Date.now();
            notificationOptions = $.extend({}, defaultNotificationOptions, notificationOptions);

            var notificationData = {};
            notificationData[notificationID] = data;
            saveNotificationData(notificationData);

            chrome.notifications.getAll(function (openNotifications) {
                var notificationActionCallback = function () {
                    if (settings && settings.notificationSound) {
                        console.logv(chrome.runtime.getURL('assets/audio/' + settings.notificationSound + '.mp3'));
                        setTimeout(function () {
                            var notificationAudio = new Audio();
                            notificationAudio.src = chrome.runtime.getURL('assets/audio/' + settings.notificationSound + '.mp3');
                            notificationAudio.play();
                        }, 1);
                    }

                    lastNotificationTabID = tabID;
                    lastNotificationTimestamp = new Date();
                };

                if (forceNotification || !openNotifications[notificationID || 'defaultNotification']) {
                    chrome.notifications.create((notificationID || 'defaultNotification'), notificationOptions, notificationActionCallback);
                }
            });
        }
    }

    function OnNotificationButtonClick(notificationID, buttonIndex) {
        chrome.notifications.clear(notificationID, function () { });

        var urlPatternPrefix = '*://*';

        if (notificationID.startsWith('standUpNotification')) {
            if (buttonIndex === -1) {
                chrome.tabs.create({ url: chrome.runtime.getURL('options.html') }, function () { });
            }
        }
        else if (notificationID.startsWith('generalNotification')) {
            //if (buttonIndex === -1) {
            getNotificationData(function (notificationData) {
                if (notificationData[notificationID] && notificationData[notificationID].link) {
                    if (notificationData[notificationID].isExternalUrl) {
                        chrome.tabs.create({ url: notificationData[notificationID].link }, function () { });
                    }
                    else {
                        var subUrl = notificationData[notificationID].link;
                        Omnibox.navigateTab(subUrl, 'https', 'newForegroundTab', '*' + subUrl);
                    }
                }
            });
            //}
        }
        else if (notificationID.startsWith('supportRequestNotification')) {
            if (buttonIndex === -1) { // Notification click
                Omnibox.navigateTab('/SupportCenter/PersonIssueSupportRequests.aspx', 'https', 'newForegroundTab', '*/PersonIssueSupportRequests.aspx', true);
            }
            else if (buttonIndex === 0) { // View Issue button
                getNotificationData(function (notificationData) {
                    var issueID = notificationData[notificationID].issueID;
                    var subUrl = '/SupportCenter/IssueEdit.aspx?IssueID=' + issueID;
                    Omnibox.navigateTab(subUrl, 'https', 'newForegroundTab', '*' + subUrl);
                });
            }
            else if (buttonIndex === 1) { // Snooze button
                Settings.get(function (settings) {
                    if (settings.enabled
                        && settings.notificationsEnabled
                        && settings.supportRequestNotificationsEnabled) {

                        var snoozedNotificationID = snoozedString + notificationID;
                        getNotificationData(function (notificationData) {
                            notificationData[snoozedNotificationID] = notificationData[notificationID];
                            delete notificationData[notificationID];
                            saveNotificationData(notificationData);
                        });

                        console.logv('Creating alarm: ' + snoozedNotificationID);
                        var tenMinutes = 1000 * 60 * 10;
                        chrome.alarms.create(snoozedNotificationID, { 'when': Date.now() + tenMinutes });
                    }
                });
            }
        }
    }

    function OnNotificationClosed(notificationID, byUser) {
        setTimeout(function () {
            getNotificationData(function (notificationData) {
                delete notificationData[notificationID];
                saveNotificationData(notificationData);
            });
        }, 1000);
    }

    function getNotificationData(responseCallback) {
        chrome.storage.sync.get(['notificationData'], function (data) {
            console.logv('pulled from sync');
            console.logv(data);
            var notificationData = $.extend({}, defaultNotificationData, data.notificationData);
            console.logv('default options');
            console.logv(defaultNotificationData);
            console.logv('pulled from sync and extended');
            console.logv(notificationData);
            responseCallback(notificationData);
        });
    }

    function saveNotificationData(notificationData, responseCallback) {
        console.logv('push to sync');
        console.logv(notificationData);

        chrome.storage.sync.set({ 'notificationData': notificationData }, function () {
            var err = chrome.runtime.lastError;

            if (err) { console.err(err); }

            if (err && responseCallback) {
                responseCallback({ 'success': false, 'errorMessage': err });
            }
            else if (responseCallback) {
                responseCallback({ 'success': true, 'errorMessage': null });
            }
        });
    }

    function sendOneWayMessageToContentScript(message) {
        chrome.tabs.query({ 'url': '*://*.pioneerrx.com/*' }, function (tabs) {
            if (tabs && tabs.length > 0) {
                for (var i = 0; i < tabs.length; i++) {
                    console.logv('sending message to tab id: ' + tabs[i].id);

                    chrome.tabs.sendMessage(tabs[i].id, message, function () {
                        if (chrome.runtime.lastError) {
                            console.log('error sending to tab:');
                            console.log(chrome.runtime.lastError);
                        }
                    });
                }
            }
        });
    }

    chrome.notifications.onClosed.addListener(OnNotificationClosed);
    chrome.notifications.onButtonClicked.addListener(OnNotificationButtonClick);
    chrome.notifications.onClicked.addListener(
        function (notificationID, buttonIndex) {
            OnNotificationButtonClick(notificationID, -1);
        }
    );

    return {
        raise: raiseNotification,
        raiseSupportRequest: raiseSupportRequestNotification,
        sendMessageToContentScript: sendOneWayMessageToContentScript
    };
})();
Object.freeze(Notification);