var Omnibox = (function (exports) {
    var searchTerms = [
        'calendar',
        'ds',
        'myissues',
        'issues',
        'help',
        'locations',
        'salesorders',
        'salesorder'
    ];
    Object.freeze(searchTerms);

    function navigateOmniTab(url, proto, disposition, existingTabPattern, reload) {
        var chromePrefix = 'chrome-extension://';

        if (url.length < chromePrefix.length || url.substring(0, chromePrefix.length) !== chromePrefix) {
            proto = proto ? proto : 'http';
            url = proto + '://www.pioneerrx.com' + url;
        }

        if (existingTabPattern) {
            existingTabPattern = '*://*.pioneerrx.com/' + existingTabPattern;

            focusTabByUrlPattern(existingTabPattern, function () {
                openOmniTab(url, disposition);
            });
        }
        else {
            openOmniTab(url, disposition);
        }
    }

    function openOmniTab(url, disposition, existingTabPattern) {
        if (disposition === 'currentTab') {
            chrome.tabs.update(undefined, { url: url, active: true }, function () { });
        }
        else {
            focusTabByUrlPattern(existingTabPattern || url, function () {
                var isActive = disposition == 'newForegroundTab';
                chrome.tabs.create({ url: url, active: isActive }, function () { });
            });
        }
    }

    function focusTabByUrlPattern(urlPattern, fn_notFound) {
        setTimeout(function () {
            chrome.tabs.query({ url: urlPattern }, function (tabResults) {
                if (tabResults && tabResults.length > 0) {
                    focusTab(tabResults[0]);
                }
                else {
                    if (typeof fn_notFound == 'function') {
                        fn_notFound();
                    }
                }
            });
        }, 200);
    }

    function focusTab(tab, url) {
        chrome.windows.update(tab.windowId, { focused: true }, function (updatedWindow) {
            var options = { active: true };
            if (url) {
                options.url = url;
            }
            chrome.tabs.update(tab.id, options, function () {
                if (url) {
                    chrome.tabs.reload(tab.id);
                }
            });
        });
    }

    chrome.omnibox.onInputChanged.addListener(function (text, suggest) {
        var suggestions = [];
        if (text.length <= 1 || 'calendar'.substring(0, Math.min(text.length, 8)) == text.substring(0, Math.min(text.length, 8))) {
            suggestions.push({ content: 'calendar', description: 'calendar <dim>Go directly to the calendar</dim>' });
        }
        if (text.length <= 1 || 'daily status'.substring(0, Math.min(text.length, 3)) == text.substring(0, Math.min(text.length, 3))) {
            suggestions.push({ content: 'daily status', description: 'daily status <dim>Go directly to Daily Status or open by typing "ds"</dim>' });
        }
        if (text.length <= 1 || 'myissues'.substring(0, Math.min(text.length, 5)) == text.substring(0, Math.min(text.length, 5))) {
            suggestions.push({ content: 'myissue', description: 'myissue <dim>Go to My Issues by typing "myissues"</dim>' });
        }
        if (text.length <= 1 || 'issue'.substring(0, Math.min(text.length, 5)) == text.substring(0, Math.min(text.length, 5))) {
            suggestions.push({ content: 'issue', description: 'issue <dim>Edit an issue by typing "issue 123456" or go to Issue search with "issue"</dim>' });
        }
        if (text.length <= 1 || 'issues'.substring(0, Math.min(text.length, 11)) == text.substring(0, Math.min(text.length, 11))) {
            suggestions.push({ content: 'issues', description: 'issues <dim>Go to Issue Search</dim>' });
        }
        if (text.length <= 1 || 'locations'.substring(0, Math.min(text.length, 9)) == text.substring(0, Math.min(text.length, 9))) {
            suggestions.push({ content: 'locations', description: 'locations <dim>Go to Location Search by typing "locations" or edit a location like [location LocationID]</dim>' });
        }
        if (text.length <= 1 || 'salesorder'.substring(0, Math.min(text.length, 11)) == text.substring(0, Math.min(text.length, 11))) {
            suggestions.push({ content: 'salesorder', description: 'salesorder <dim>Edit a Sales Order by typing "salesorder 123456" or go to Sales Order search with "salesorder"</dim>' });
        }
        if (text.length <= 1 || 'salesorders'.substring(0, Math.min(text.length, 11)) == text.substring(0, Math.min(text.length, 11))) {
            suggestions.push({ content: 'salesorders', description: 'salesorders <dim>Go to Sales Order Search</dim>' });
        }
        if (text.length <= 1 || 'help'.substring(0, Math.min(text.length, 4)) == text.substring(0, Math.min(text.length, 4)) ||
            suggestions.length === 0) {
            suggestions.push({ content: 'help', description: 'Help! <dim>What does this do?</dim>' });
        }

        chrome.omnibox.setDefaultSuggestion({ description: suggestions[suggestions.length - 1].description });

        suggestions.shift();

        suggest(suggestions);
    });

    chrome.omnibox.onInputEntered.addListener(function (text, disposition) {
        for (var i = 0; i < searchTerms.length; i++) {
            if (searchTerms[i].length >= text.trim().length &&
                searchTerms[i].substring(0, text.trim().length) == text.trim().toLowerCase()) {
                var otherMatch = false;
                for (var j = 0; j < searchTerms.length; j++) {
                    if (j !== i && searchTerms[j].length >= text.trim().length &&
                        searchTerms[j].substring(0, text.trim().length) == text.trim().toLowerCase()) {
                        otherMatch = true;
                        console.logv(text + ' was ambiguous. matched "' + searchTerms[i] + '" and "' + searchTerms[j] + '"');
                    }
                }
                if (!otherMatch) {
                    console.logv('replacing search "' + text + '" with "' + searchTerms[i] + '"');
                    text = searchTerms[i];
                }
                break;
            }
        }

        console.logv('searching for: ' + text);

        // daily status
        if (/^(ds|da|dai|dail|daily|daily s|daily st|daily sta|daily stat|daily statu|daily status|status)/i.test(text)) {
            navigateOmniTab('/SupportCenter/DailyStatusListForPerson.aspx', 'http', disposition, '*/DailyStatusListForPerson.aspx');
        }
        // calendar
        else if (/^cal(e|en|end|enda|endar)/i.test(text)) {
            navigateOmniTab('/SupportCenter/Calendar/', 'https', disposition, '*/Calendar/');
        }
        // issues
        else if (/^i(s|ss|ssu|ssue|ssues)? ([0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12})/i.test(text)) {
            var match = /^i(s|ss|ssu|ssue|ssues)? ([0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12})/i.exec(text);
            console.logv(match);
            navigateOmniTab('/SupportCenter/IssueEdit.aspx?IssueID=' + match[2], 'https', disposition, '*/IssueEdit.aspx?IssueID=' + match[2]);
        }
        else if (/^i(s|ss|ssu|ssue|ssues)? ([0-9]+)/i.test(text)) {
            var match = /^i(s|ss|ssu|ssue|ssues)? ([0-9]+)/i.exec(text);
            console.logv(match);
            var issueNumber = match[2];
            navigateOmniTab('/SupportCenter/IssueEdit.aspx?IssueNumber=' + issueNumber, 'https', disposition, '*/IssueEdit.aspx?IssueNumber=' + issueNumber);
        }
        else if (/^(my|my issue(s)?|myissue(s)?)/i.test(text)) {
            navigateOmniTab('/SupportCenter/IssueMyListAdvanced.aspx', 'https', disposition, '*/IssueMyListAdvanced.aspx');
        }
        else if (/^(issues|issue search|issuesearch|search issues)|(q)+?/i.test(text)) {
            navigateOmniTab('/SupportCenter/IssueSearchAdvanced.aspx', 'https', disposition, '*/IssueSearchAdvanced.aspx');
        }
        // locations
        else if (/^l(|o|oc|oca|ocat|ocati|ocatio|ocation|ocations)? ([0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12})/i.test(text)) {
            var match = /^l(|o|oc|oca|ocat|ocati|ocatio|ocation|ocations)? ([0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12})/i.exec(text);
            console.logv(match);
            navigateOmniTab('/LocationEdit.aspx?LocationID=' + match[2], 'https', disposition, 'LocationEdit.aspx?LocationID=' + match[2]);
        }
        else if (/^locations/i.test(text)) {
            navigateOmniTab('/LocationSearch.aspx', 'https', disposition, 'LocationSearch.aspx');
        }
        // sales orders
        else if (/^s(o|ales|alesorder|ales order)? ([0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12})/i.test(text)) {
            var match = /^s(o|ales|alesorder|ales order)? ([0-9a-f]{8}-[0-9a-f]{4}-[1-5][0-9a-f]{3}-[89ab][0-9a-f]{3}-[0-9a-f]{12})/i.exec(text);
            console.logv(match);
            navigateOmniTab('/QBIntegration/SalesOrderEdit.aspx?SalesOrderID=' + match[2], 'https', disposition, '*/SalesOrderEdit.aspx?SalesOrderID=' + match[2]);
        }
        else if (/^s(o|ales|alesorder|ales order)? ([0-9]+)/i.test(text)) {
            var match = /^s(o|ales|alesorder|ales order)? ([0-9]+)/i.exec(text);
            console.logv(match);
            var salesOrderNumber = match[2];
            navigateOmniTab('/QBIntegration/SalesOrderEdit.aspx?SalesOrderNumber=' + salesOrderNumber, 'https', disposition, '*/SalesOrderEdit.aspx?SalesOrderNumber=' + salesOrderNumber);
        }
        else if (/^s(o|ales|alesorder|ales order)/i.test(text)) {
            navigateOmniTab('/QBIntegration/SalesOrderSearch.aspx', 'https', disposition, '*/SalesOrderSearch.aspx');
        }
        else if (/^(call|calls|support|supportcalls|support calls)/i.test(text)) {
            navigateOmniTab('/SupportCenter/IssueCallList.aspx', 'https', disposition, '*/IssueCallList.aspx');
        }
        // default to help page
        else {
            navigateOmniTab(chrome.runtime.getURL('omniboxhelp.html'), null, disposition);
        }
    });

    return {
        navigateTab: navigateOmniTab,
        openTab: openOmniTab
    };

})();
Object.freeze(Omnibox);