(function (exports) {
    var one_second = 1000;
    var currentSettings;

    // Saves options to chrome.storage
    function save_options() {
        var dsQuickAdd = document.getElementById('chkDailyStatusQuickAdd').checked;
        var employeeSearch = document.getElementById('chkEmployeeSearch').checked;

        var settings = {
            showDailyStatusQuickAdd: $('#chkDailyStatusQuickAdd').prop('checked'),
            showEmployeeSearch: $('#chkEmployeeSearch').prop('checked'),
            notificationsEnabled: $('#chkNotificationsEnabled').prop('checked'),
            supportRequestNotificationsEnabled: $('#chkSupportRequestNotificationsEnabled').prop('checked'),
            newIssueNotificationsEnabled: $('#chkNewIssueNotificationsEnabled').prop('checked'),
            standUpNotificationsEnabled: $('#chkStandUpNotificationsEnabled').prop('checked'),
            standUpNotificationInterval: parseInt($('#txtStandUpNotificationTimeout').val() || 45),
            notificationSound: $('#drpSound').val()
        };

        settings = $.extend({}, currentSettings, settings);

        chrome.runtime.sendMessage({ eventName: Events.Settings.Save, settings: settings }, function (response) {
            if (response && response.success != undefined && (response.success || response.errorMessage)) {
                if (response.success) {
                    $('#soundHint').addClass('hide');

                    TOAST.success('Preferences saved');

                    if (settings
                        && settings.enabled
                        && settings.notificationsEnabled
                        && settings.standUpNotificationsEnabled) {
                        var interval = settings.standUpNotificationInterval || 45;
                        chrome.alarms.create('standUpAlarm', { delayInMinutes: interval, periodInMinutes: interval });
                    }
                    else {
                        chrome.alarms.clear('standUpAlarm', function () {
                            console.logv('Cleared alarm standUpAlarm' + (settings.standUpNotificationsEnabled ? ' -- notification disabled (Settings)' : ''));
                        });
                    }
                }
                else {
                    TOAST.error('Error saving: ' + response.errorMessage);
                }
            }
            else {
                TOAST.error('Unknown error saving');
            }
        }
        );
    }

    // Restores select box and checkbox state using the preferences stored in chrome.storage.
    function restore_options() {
        chrome.runtime.sendMessage({ eventName: Events.Settings.Get }, function (settings) {
            currentSettings = settings;

            $('#chkDailyStatusQuickAdd').prop('checked', settings.showDailyStatusQuickAdd);
            $('#chkEmployeeSearch').prop('checked', settings.showEmployeeSearch);

            $('#chkNotificationsEnabled').prop('checked', settings.notificationsEnabled);
            $('#chkSupportRequestNotificationsEnabled').prop('checked', settings.supportRequestNotificationsEnabled);
            $('#chkNewIssueNotificationsEnabled').prop('checked', settings.newIssueNotificationsEnabled);
            $('#chkStandUpNotificationsEnabled').prop('checked', settings.standUpNotificationsEnabled);
            $('#txtStandUpNotificationTimeout').val(settings.standUpNotificationInterval);
            $('#drpSound').val(settings.notificationSound);

            // init toggles
            $('input[type="checkbox"]').bootstrapToggle();

            toggleNotifications();
            $('#chkNotificationsEnabled').change(toggleNotifications);
        }
        );
    }

    function toggleNotifications() {
        var notificationsEnabled = document.getElementById('chkNotificationsEnabled').checked;
        $('#notifications .sub-toggle input[type="checkbox"]').bootstrapToggle(notificationsEnabled ? 'enable' : 'disable');
        $('#testNotification').prop('disabled', !notificationsEnabled);
        $('#drpSound').prop('disabled', !notificationsEnabled);
    }

    function test_notification() {
        try {
            chrome.runtime.sendMessage({
                eventName: Events.Notifications.RaiseNotification,
                notification: {
                    ID: 'test_notificationID',
                    options: {
                        title: 'Test Notification',
                        message: 'Testing...testing...1...2...'
                    },
                    data: null
                }
            }, null);
        }
        catch (ex) {
        }
    }

    function check_notification_permission() {
        var hasPermission = false;
        // TODO: check and see if person has permission to use extension specific notifications
        if (hasPermission) {
            var notificationSection = $("#notificationSection");
            notificationSection.fadeIn();
        }
    }

    document.addEventListener('DOMContentLoaded', restore_options);
    document.addEventListener('DOMContentLoaded', check_notification_permission);
    document.getElementById('save').addEventListener('click', save_options);
    document.getElementById('testNotification').addEventListener('click', test_notification);
    document.getElementById('drpSound').addEventListener('change', function () { $('#soundHint').toggleClass('hide'); });
})(window);