var SignalR = (function () {
	var debugUrl = 'https://prxafausett.pioneerrx.com/pioneerrxinternet/signalr/hubs';
	var productionUrl = 'https://www.pioneerrx.com/signalr/hubs';
	var hubConnection;
	var connectionHubProxy;
	var notificationHubProxy;
	var isConnected = false;
	
	function closeConnection(callback) {
		try {
			hubConnection
				.stop()
				.done(function () {
					hubConnection
						.close()
						.done(function () {
							if (callback && typeof(callback) == "function") {
								callback();
							}
						});
				});
		}
		catch (ex) { 
			if (callback && typeof(callback) == "function") {
				callback();
			}
		}
		hubConnection = null;
		isConnected = false;
	}
	
	function attemptConnection() {
		closeConnection(function () {
			Settings.get(function (settings) {
				if (settings && settings.enabled && settings.securityToken && settings.signalrEnabled) {
					var joinGroup = function () {
						if (settings && settings.enabled && settings.securityToken && hubConnection) {
							try {
								connectionHubProxy.invoke('joinGroupWithToken', settings.securityToken)
									.done(function () { isConnected = true; console.log('SignalR - joined group with token'); })
									.fail(function (err) { isConnected = false; console.error('SignalR - failed to join group', err); });
							}
							catch (ex) { }
						}
					};

					hubConnection = $.hubConnection((useProduction ? productionUrl : debugUrl), { useDefaultPath: false });
					connectionHubProxy = hubConnection.createHubProxy('connectionHub');
					notificationHubProxy = hubConnection.createHubProxy('notificationHub');
					
					notificationHubProxy.on('supportRequestNotification', function (data) {
						console.logv('supportRequestNotification message recieved');
						Settings.get(function (settings) {
							if (settings 
								&& settings.enabled
								&& settings.notificationsEnabled 
								&& settings.supportRequestNotificationsEnabled) {

								var notificationID = 'supportRequestNotification';
                                Notification.raiseSupportRequest(notificationID, data, settings);
							}
						});
					});
					
					notificationHubProxy.on('generalNotification', function (notificationData) {
						var notificationTypes = {
							General: 1,
							NewIssue: 2
						};
						
						console.logv('generalNotification message recieved');
						notificationData = notificationData || {};
						Settings.get(function (settings) {
							if (settings 
								&& settings.enabled
								&& (settings.notificationsEnabled || notificationData.forceNotification)) {

								if (!notificationData.forceNotification) {
									switch(notificationData.notificationType) {
										case notificationTypes.General:
										break;
										case notificationTypes.NewIssue:
											if (!settings.newIssueNotificationsEnabled)
												return;
										break;
									}
								}
								
								var notificationID = 'generalNotification';
								var notificationOptions = notificationData.notificationOptions;
								
                                Notification.raise(notificationID, notificationOptions, notificationData.data, null, settings, notificationData.forceNotification);
							}
						});
					});
				
					var attemptReconnect = null;
					
					hubConnection.connectionSlow(function () {
						console.info('Currently experiencing difficulties with the connection - Slow');
					});
					
					hubConnection.disconnected(function () {
						isConnected = false;
						console.log('SignalR - disconnected');
						
						attemptReconnect = setTimeout(function () {
							attemptConnection();
						}, 1000 * 150); // 150 seconds
					});
					
					hubConnection.reconnected(function () {
						if (attemptReconnect) {
							clearTimeout(attemptReconnect);
							attemptReconnect = null;
						}
						
						console.log('SignalR - reconnected', hubConnection.transport.name);
						joinGroup();
					});
					
					hubConnection.start()
						.done(function () {
							console.log('SignalR - connection started', hubConnection.transport.name);
							joinGroup();
						})
						.fail(function () {
							console.warn('SignalR - Could not connect');
						});
				}
				else {
					isConnected = false;
					console.warn('Could not start SignalR connection - user not logged in or SignalR is disabled.');
				}
			});
		});
	}
	
	return {
		isConnected: function (value) {
			if (typeof value !== 'undefined') {
				isConnected = value || isConnected;
			}
			
			return isConnected;
		},
		startConnection: attemptConnection,
		stopConnection: closeConnection,
		hubConnectoin: hubConnection
	};
})();
Object.freeze(SignalR);